@@ -1,5 +1,6 @@
 use std::fs::File;
 use std::io::Read;
+use std::str::FromStr;
 
 use chrono::{Datelike, FixedOffset as ChronoFixedOffset, NaiveDate, NaiveDateTime, Timelike, Utc as ChronoUtc};
 use strum::EnumMessage;
@@ -33,11 +34,37 @@ macro_rules! expect_ok_or_error {
     };
 }
 
+/// macro for comparing a type's `from_str` and `parse_str` methods
+macro_rules! expect_fromstr_matches_parse_str {
+    ($type:ty, $name:ident, ok, $_expected:expr, $example:expr) => {
+        paste::item! {
+            #[test]
+            fn [< expect_ $name _fromstr_matches_parse_str_ok >]() {
+                expect_fromstr_matches_parse_str!(@body $type, $example);
+            }
+        }
+    };
+    ($type:ty, $name:ident, err, $error:expr, $example:expr) => {
+        paste::item! {
+            #[test]
+            fn [< expect_ $name _fromstr_matches_parse_str_ $error:snake _error >]() {
+                expect_fromstr_matches_parse_str!(@body $type, $example);
+            }
+        }
+    };
+    (@body $type:ty, $example:expr) => {{
+        let fromstr = <$type>::from_str($example);
+        let parse_str = <$type>::parse_str($example);
+        assert_eq!(fromstr, parse_str, "fromstr: {fromstr:?}, parse_str: {parse_str:?}");
+    }};
+}
+
 /// macro to define many tests for expected values
 macro_rules! param_tests {
     ($type:ty, $($name:ident: $ok_or_err:ident => $input:literal, $expected:expr;)*) => {
         $(
             expect_ok_or_error!($type, $name, $ok_or_err, $input, $expected);
+            expect_fromstr_matches_parse_str!($type, $name, $ok_or_err, $expected, $input);
         )*
     }
 }
@@ -203,7 +230,7 @@ fn date_comparison() {
 }
 
 #[test]
-fn date_timestamp() {
+fn date_timestamp_exact() {
     let d = Date::from_timestamp(1_654_560_000, true).unwrap();
     assert_eq!(d.to_string(), "2022-06-07");
     assert_eq!(d.timestamp(), 1_654_560_000);
@@ -212,6 +239,16 @@ fn date_timestamp() {
         Ok(d) => panic!("unexpectedly valid, {d}"),
         Err(e) => assert_eq!(e, ParseError::DateNotExact),
     }
+
+    // milliseconds
+    let d = Date::from_timestamp(1_654_560_000_000, true).unwrap();
+    assert_eq!(d.to_string(), "2022-06-07");
+    assert_eq!(d.timestamp(), 1_654_560_000);
+
+    match Date::from_timestamp(1_654_560_000_001, true) {
+        Ok(d) => panic!("unexpectedly valid, {d}"),
+        Err(e) => assert_eq!(e, ParseError::DateNotExact),
+    }
 }
 
 macro_rules! date_from_timestamp {
@@ -722,6 +759,14 @@ fn datetime_tz_negative_2212() {
     let dt = DateTime::parse_str("2020-01-01T12:13:14−02:15").unwrap();
     assert_eq!(dt.time.tz_offset, Some(-8100));
     assert_eq!(dt.to_string(), "2020-01-01T12:13:14-02:15");
+
+    let dt = DateTime::parse_str("2020-01-01T12:13:14−00:01").unwrap();
+    assert_eq!(dt.time.tz_offset, Some(-60));
+    assert_eq!(dt.to_string(), "2020-01-01T12:13:14-00:01");
+
+    let dt = DateTime::parse_str("2020-01-01T12:13:14−01:00").unwrap();
+    assert_eq!(dt.time.tz_offset, Some(-3600));
+    assert_eq!(dt.to_string(), "2020-01-01T12:13:14-01:00");
 }
 
 #[test]
@@ -818,7 +863,18 @@ param_tests! {
     dt_underscore: ok => "2020-01-01_12:13:14,123z", "2020-01-01T12:13:14.123000Z";
     dt_unix1: ok => "1654646400", "2022-06-08T00:00:00";
     dt_unix2: ok => "1654646404", "2022-06-08T00:00:04";
+    dt_unix_1_neg: ok => "-1654646400", "1917-07-27T00:00:00";
+    dt_unix_2_neg: ok => "-1654646404", "1917-07-26T23:59:56";
     dt_unix_float: ok => "1654646404.5", "2022-06-08T00:00:04.500000";
+    dt_unix_float_limit: ok => "1654646404.123456", "2022-06-08T00:00:04.123456";
+    dt_unix_float_ms: ok => "1654646404000.5", "2022-06-08T00:00:04.000500";
+    dt_unix_float_ms_limit: ok => "1654646404123.456", "2022-06-08T00:00:04.123456";
+    dt_unix_float_ms_neg: ok => "-1654646404.123456", "1917-07-26T23:59:55.876544";
+    dt_unix_float_ms_neg_limit: ok => "-1654646404000.123", "1917-07-26T23:59:55.999877";
+    dt_unix_float_empty: ok => "1654646404.", "2022-06-08T00:00:04";
+    dt_unix_float_ms_empty: ok => "1654646404000.", "2022-06-08T00:00:04";
+    dt_unix_float_too_long: err => "1654646404.1234567", SecondFractionTooLong;
+    dt_unix_float_ms_too_long: err => "1654646404123.4567", MillisecondFractionTooLong;
     dt_short_date: err => "xxx", TooShort;
     dt_short_time: err => "2020-01-01T12:0", TooShort;
     dt: err => "202x-01-01", InvalidCharYear;
@@ -858,7 +914,7 @@ fn test_ok_values_txt() {
     let mut success = 0;
     for (i, line) in contents.split('\n').enumerate() {
         let line_no = i + 1;
-        if line.starts_with('#') || line.is_empty() {
+        if line.starts_with('#') || line.is_empty() || line == "\r" {
             continue;
         } else if line.starts_with("date:") {
             let (input, expected_str) = extract_values(line, "date:");
@@ -1053,6 +1109,40 @@ fn duration_new_err() {
     }
 }
 
+#[test]
+fn duration_hours() {
+    let d = Duration::parse_str("PT5H45M").unwrap();
+    assert_eq!(
+        d,
+        Duration {
+            positive: true,
+            day: 0,
+            second: 20700,
+            microsecond: 0
+        }
+    );
+    assert_eq!(d.to_string(), "PT5H45M");
+    assert_eq!(d.signed_total_seconds(), (5 * 60 * 60) + (45 * 60));
+    assert_eq!(d.signed_microseconds(), 0);
+}
+
+#[test]
+fn duration_minutes() {
+    let d = Duration::parse_str("PT30M").unwrap();
+    assert_eq!(
+        d,
+        Duration {
+            positive: true,
+            day: 0,
+            second: 1800,
+            microsecond: 0
+        }
+    );
+    assert_eq!(d.to_string(), "PT30M");
+    assert_eq!(d.signed_total_seconds(), 30 * 60);
+    assert_eq!(d.signed_microseconds(), 0);
+}
+
 param_tests! {
     Duration,
     duration_too_short1: err => "", TooShort;
@@ -1064,14 +1154,14 @@ param_tests! {
     duration_1m: ok => "P1M", "P30D";
     duration_1_5m: ok => "P1.5M", "P45D";
     duration_1w: ok => "P1W", "P7D";
-    duration_1_1w: ok => "P1.1W", "P7DT60480S";
-    duration_1_123w: ok => "P1.123W", "P7DT74390.4S";
+    duration_1_1w: ok => "P1.1W", "P7DT16H48M";
+    duration_1_123w: ok => "P1.123W", "P7DT20H39M50.4S";
     duration_simple_negative: ok => "-P1Y", "-P1Y";
     duration_simple_positive: ok => "+P1Y", "P1Y";
     duration_fraction1: ok => "PT0.555555S", "PT0.555555S";
-    duration_fraction2: ok => "P1Y1DT2H0.5S", "P1Y1DT7200.5S";
+    duration_fraction2: ok => "P1Y1DT2H0.5S", "P1Y1DT2H0.5S";
     duration_1: ok => "P1DT1S", "P1DT1S";
-    duration_all: ok => "P1Y2M3DT4H5M6S", "P1Y63DT14706S";
+    duration_all: ok => "P1Y2M3DT4H5M6S", "P1Y63DT4H5M6S";
     duration: err => "PD", DurationInvalidNumber;
     duration: err => "P1DT1MT1S", DurationTRepeated;
     duration: err => "P1DT1.1M1S", DurationInvalidFraction;
@@ -1079,12 +1169,18 @@ param_tests! {
     duration_invalid_day_unit1: err => "P1X", DurationInvalidDateUnit;
     duration_invalid_day_unit2: err => "P1", DurationInvalidDateUnit;
     duration_time_42s: ok => "00:00:42", "PT42S";
-    duration_time_1m: ok => "00:01", "PT60S";
-    duration_time_1h_2m_3s: ok => "01:02:03", "PT3723S";
-    duration_time_fraction: ok => "00:01:03.123", "PT63.123S";
+    duration_time_1m: ok => "00:01", "PT1M";
+    duration_time_1h_2m_3s: ok => "01:02:03", "PT1H2M3S";
+    duration_time_fraction: ok => "00:01:03.123", "PT1M3.123S";
     duration_time_extra: err => "00:01:03.123x", ExtraCharacters;
     duration_time_timezone: err => "00:01:03x", ExtraCharacters;
-    duration_time_invalid_hour: err => "24:01:03", OutOfRangeHour;
+    duration_time_more_than_24_hour: ok => "24:01:03", "P1DT1M3S";
+    duration_time_way_more_than_24_hour: ok => "2400000000:01:03", "P273972Y220DT1M3S";
+    duration_time_way_more_than_24_hour_long_fraction: ok => "2400000000:01:03.654321", "P273972Y220DT1M3.654321S";
+    duration_time_invalid_over_limit_hour: err => "100000000000:01:03", DurationHourValueTooLarge;
+    duration_time_overflow_hour: err => "100000000000000000000000:01:03", DurationHourValueTooLarge;
+    duration_time_invalid_format_hour: err => "1000xxx000:01:03", InvalidCharHour;
+    duration_time_invalid_format_hour2: err => "1 10:10", InvalidCharHour;
     duration_time_invalid_minute: err => "00:60:03", OutOfRangeMinute;
     duration_time_invalid_second: err => "00:00:60", OutOfRangeSecond;
     duration_time_fraction_too_long: err => "00:00:00.1234567", SecondFractionTooLong;
@@ -1109,7 +1205,7 @@ param_tests! {
     duration_days_123days: ok => "123days", "P123D";
     duration_days_time: ok => "1 day 00:00:42", "P1DT42S";
     duration_days_time_neg: ok => "-1 day 00:00:42", "-P1DT42S";
-    duration_exceeds_day: ok => "PT86500S", "P1DT100S";
+    duration_exceeds_day: ok => "PT86500S", "P1DT1M40S";
     duration_days_time_too_short: err => "1 day 00:", TooShort;
     duration_days_time_wrong: err => "1 day 00:xx", InvalidCharMinute;
     duration_days_time_extra: err => "1 day 00:00:00.123 ", InvalidCharTzSign;
@@ -1133,14 +1229,14 @@ fn duration_large() {
 #[test]
 fn duration_limit() {
     let d = Duration::new(true, 999_999_999, 86399, 999_999).unwrap();
-    assert_eq!(d.to_string(), "P2739726Y9DT86399.999999S");
+    assert_eq!(d.to_string(), "P2739726Y9DT23H59M59.999999S");
 
     match Duration::new(true, 999_999_999, 86399, 999_999 + 1) {
         Ok(t) => panic!("unexpectedly valid -> {t:?}"),
         Err(e) => assert_eq!(e, ParseError::DurationDaysTooLarge),
     }
     let d = Duration::new(false, 999_999_999, 86399, 999_999).unwrap();
-    assert_eq!(d.to_string(), "-P2739726Y9DT86399.999999S");
+    assert_eq!(d.to_string(), "-P2739726Y9DT23H59M59.999999S");
 
     match Duration::new(false, 999_999_999, 86399, 999_999 + 1) {
         Ok(t) => panic!("unexpectedly valid -> {t:?}"),
@@ -1306,7 +1402,10 @@ fn test_datetime_parse_bytes_does_not_add_offset_for_rfc3339() {
 fn test_datetime_parse_unix_timestamp_from_bytes_with_utc_offset() {
     let time = DateTime::parse_bytes_with_config(
         "1689102037.5586429".as_bytes(),
-        &(TimeConfigBuilder::new().unix_timestamp_offset(Some(0)).build()),
+        &(TimeConfigBuilder::new()
+            .unix_timestamp_offset(Some(0))
+            .microseconds_precision_overflow_behavior(MicrosecondsPrecisionOverflowBehavior::Truncate)
+            .build()),
     )
     .unwrap();
     assert_eq!(time.to_string(), "2023-07-11T19:00:37.558643Z");
@@ -1316,7 +1415,10 @@ fn test_datetime_parse_unix_timestamp_from_bytes_with_utc_offset() {
 fn test_datetime_parse_unix_timestamp_from_bytes_as_naive() {
     let time = DateTime::parse_bytes_with_config(
         "1689102037.5586429".as_bytes(),
-        &(TimeConfigBuilder::new().unix_timestamp_offset(None).build()),
+        &(TimeConfigBuilder::new()
+            .unix_timestamp_offset(None)
+            .microseconds_precision_overflow_behavior(MicrosecondsPrecisionOverflowBehavior::Truncate)
+            .build()),
     )
     .unwrap();
     assert_eq!(time.to_string(), "2023-07-11T19:00:37.558643");
